--[[
This file is part of Exploding numbers - a puzzle game

Exploding numbers is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
]]

--[[
local function c(h, a)
	local r, g, b =
		tonumber(h:sub(1, 2), 16),
		tonumber(h:sub(3, 4), 16),
		tonumber(h:sub(5, 6), 16)
	local function round(n)
		return tostring(
			-- round to 3 decimal places
			math.floor(n * 1000 / 255 + .5) / 1000
		):gsub('^0*(.-)0*$', '%1') -- trim leading and trailing 0s
	end
	print(
		'Loaded color #' .. h .. ' as {' ..
		round(r) .. ', ' .. round(g) .. ', ' .. round(b) .. '}'
	)
	return {love.math.colorFromBytes(r, g, b, a)}
end


local function dump_c(color_scheme)
	print("title = '" .. color_scheme.title .. "',")
	for k, v in pairs(color_scheme) do if k ~= 'title' then
		local o = k .. " = {"
		for a = 1, #v do
			if a > 1 then o = o .. ', ' end
			o = o .. math.floor(v[a] * 1000 + .5) / 1000
		end
		print(o .. '},')
	end end
end
]]

colors = {
	bonus = 6,
	{
		title = 'day',
		menu_bg = {.7, .6, .4},
		menu_text = {0, 0, 0},
		menu_text_disabled = {.42, .361, .24},
		bg = {.8, .8, .8},
		numbers = {0, 0, 0},
		neutral = {.6, .6, .6},
		odd = {.8, .5, .5},
		db4 = {.5, .8, .5},
		goal = {.6, .7, .9},
		selected = {1, 1, 1},
		naked_bg = {.7, .7, .7},
		naked_neutral = {.5, .5, .5},
		naked_odd = {.7, .2, .2},
		naked_db2 = {.2, .2, .2},
		naked_db4 = {.2, .6, .2},
		naked_goal = {.3, .35, .5},
		naked_indicator = {.5, .5, .5},
		heading = {0, 0, .5},
		progress = {.7, .6, .4},
	},
	{
		title = 'night',
		menu_bg = {.1, .1, .1},
		menu_text = {.75, .6, .8},
		menu_text_disabled = {.8, .5, .8, .4},
		bg = {.15, .15, .2},
		neutral = {.2, .2, .3},
		numbers = {.8, .8, .8},
		odd = {.075, .075, .1},
		db4 = {.3, .1, .2},
		goal = {.4, .5, .1},
		indicator = {.8, .8, .8, .5},
		selected = {.25, .25, .25},
		naked_neutral = {.25, .25, .38},
		naked_odd = {.55, .55, .8},
		naked_db2 = {.8, .8, .8},
		naked_db4 = {.8, .4, .6},
		naked_selected = {.4, .4, .4},
		progress = {0, 0, 0},
		heading = {1, 1, 1},
		message_bg = {.3, .1, .2},
		message_text = {.75, .6, .8},
		warning_bg = {.8, .4, .6},
		warning_text = {.1, .1, .1},
	},
	{
		title = 'vivid',
		menu_bg = {.6, .5, .7},
		menu_text = {0, 0, 0},
		menu_text_disabled = {0, 0, 0, .4},
		bg = {.23, .23, .33},
		neutral = {.5, .5, .6},
		numbers = {0, 0, 0},
		odd = {.8, .2, .2},
		db4 = {.5, .8, .4},
		goal = {.6, .4, .8},
		selected = {.77, .77, .87},
		naked_indicator = {.5, .5, .6},
		progress = {1, .7, .5},
		help_bg = {.2, .2, .3},
		help_text = {.8, .8, .8},
		heading = {.8, .9, .6},
	},
	{
		title = 'oled',
		menu_bg = {0, 0, 0},
		menu_text = {1, 1, 1},
		menu_text_disabled = {.4, .4, .4},
		bg = {0, 0, 0},
		neutral = {.4, .4, .4},
		numbers = {0, 0, 0},
		odd = {.6, .1, .1},
		db4 = {0, .6, .6},
		goal = {.8, .3, .7},
		selected = {.4, .4, .1},
		progress = {0, 0, 1},
		heading = {.6, .6, .6},
		message_bg = {1, 1, 1},
		message_text = {0, 0, 0},
		warning_bg = {0, 0, 0},
		warning_text = {1, 1, 1},
	},
	{
		title = 'cyborg',
		menu_bg = {.03, .108, .282},
		menu_text = {.1, .4, .953},
		menu_text_disabled = {.05, .308, .698},
		bg = {.043, .085, .2},
		neutral = {.043, .085, .2, 0},
		numbers = {.2, .02, .02},
		odd = {.45, .05, .05},
		db2 = {.56, .06, .06},
		db4 = {.72, .07, .07},
		goal = {0, 0, 0},
		goal_numbers = {0, 0, 1},
		indicator = {.155, .31, .72},
		selected = {.155, .31, .72, .3},
		selected_numbers = {.155, .31, .72},
		naked_neutral = {.155, .31, .72},
		naked_odd = {.56, .06, .06, .6},
		naked_db2 = {.72, .07, .07},
		naked_db4 = {.9, .1, .1},
		naked_goal = {0, 0, 1},
		naked_selected = {.155, .31, .72},
		progress = {.55, .06, .06},
		help_bg = {.043, .085, .2},
		help_text = {.155, .31, .72},
		heading = {.25, .5, 1},
		message_bg = {.282, .108, .03},
		message_text = {.953, .4, .1},
		warning_bg = {0, 0, 0},
		warning_text = {.07, .323, .953},
	},
	{
		title = 'acrylic',
		menu_bg = {.404, .533, .384},
		menu_text = {.85, .85, .85},
		menu_text_disabled = {.271, .369, .259},
		bg = {.315, .345, .39},
		neutral = {.31, .32, .385},
		numbers = {.24, .293, .345},
		odd = {.03, 0, .02},
		db2 = {.608, .584, .267},
		db4 = {.584, .604, .624},
		goal = {.404, .533, .384},
		indicator = {.03, 0, .02},
		selected = {.6, .341, .518},
		naked_neutral = {.2, .21, .27},
		naked_indicator = {.404, .533, .384},
		progress = {.62, .341, .518},
		help_bg = {.584, .604, .624},
		help_text = {.24, .293, .345},
		heading = {.03, 0, .02},
		message_text = {0, 0, 0},
		warning_bg = {.6, .341, .518},
	},
	{
		title = 'rust',
		menu_text = {.173, .196, .243},
		menu_bg = {.573, .612, .651},
		menu_text_disabled = {.757, .765, .78},
		bg = {.212, .192, .22},
		neutral = {.165, .149, .165},
		numbers = {1, 1, 1},
		odd = {.216, .212, .616},
		db2 = {.608, .267, .569},
		db4 = {.643, .275, .275},
		goal = {.3, .525, .35},
		indicator = {.384, .36, .35},
		selected = {.404, .408, .376},
		selected = {.384, .36, .35},
		naked_selected = {.906, .871, .749},
		naked_neutral = {0, 0, 0},
		naked_odd = {.345, .341, .992},
		naked_db2 = {.992, .439, .925},
		naked_db4 = {.988, .431, .431},
		naked_indicator = {.384, .36, .35},
		progress = {.643, .275, .275},
		help_text = {.757, .765, .78},
		heading = {.906, .89, .871},
		message_bg = {.573, .612, .651},
		message_text = {0, 0, 0},
		warning_bg = {.643, .275, .275},
	},
	{
		title = 'sudoku',
		menu_bg = {.65, .66, .65},
		menu_text = {0, 0, 0},
		menu_text_disabled = {0, 0, 0, .4},
		bg = {.85, .85, .85},
		numbers = {.85, .85, .85},
		odd = {0, 0, 0},
		odd_numbers = {.3, .3, .3},
		db2 = {0, 0, 0},
		db4 = {0, .42, .66},
		goal = {0, .66, .42},
		indicator = {0, 0, 0},
		selected = {.85, 0, 0},
		naked_odd = {.5, .5, .5},
		naked_indicator = {0, 0, 0},
		progress = {0, .42, .66},
		help_text = {0, 0, 0},
		heading = {0, .38, .6},
		message_bg = {0, .659, .42},
		message_text = {0, 0, 0},
		warning_bg = {.85, 0, 0},
	},
	{
		title = 'sakura',
		menu_bg = {1, .778, .713},
		menu_text = {.3, .23, .23},
		menu_text_disabled = {1, .98, .95},
		selected = {.9, .1, .1},
		bg = {.95, .85, .85},
		neutral = {1, 1, 1},
		numbers = {.95, .85, .85},
		odd = {.23, .12, .1},
		db2 = {.23, .12, .1},
		db4 = {.23, .12, .1},
		goal = {.1, .6, .3},
		naked_bg = {1, .778, .713},
		naked_indicator = {1, 1, 1},
		progress = {1, 1, 1},
		help_text = {0, 0, 0},
		heading = {.5, .15, .15},
		message_bg = {.1, .6, .3},
		message_text = {1, 1, 1},
		warning_bg = {.85, .384, .352},
	},
	{
		title = 'fire',
		menu_bg = {0.141, 0.043, 0.067},
		menu_text = {0.812, 0.329, 0.196},
		menu_text_disabled = {0.231, 0.216, 0.271},
		bg = {0.969, 0.286, 0.004},
		neutral = {0.973, 0.337, 0.035},
		numbers = {0.306, 0.043, 0.071},
		odd = {0.988, 0.337, 0.004},
		db2 = {0.973, 0.337, 0.035},
		db2_numbers = {0.98, 0.741, 0.149},
		db4 = {0.996, 0.388, 0.008},
		db4_numbers = {0.988, 0.812, 0.31},
		goal = {0.42, 0.333, 0.475},
		goal_numbers = {0.98, 0.647, 0.004},
		indicator = {0.98, 0.741, 0.149},
		selected = {0.992, 0.714, 0.012},
		selected_numbers = {0.718, 0.125, 0.071},
		naked_bg = {0.576, 0.153, 0.153},
		naked_db2 = {0.98, 0.741, 0.149},
		naked_db4 = {0.988, 0.812, 0.31},
		naked_goal = {0.141, 0.043, 0.067},
		progress = {0.843, 0.216, 0.2},
		help_bg = {0.176, 0.078, 0.129},
		heading = {0.988, 0.631, 0},
		message_bg = {0.2, 0.188, 0.302},
		message_text = {0.992, 0.365, 0.059},
		warning_bg = {0.847, 0.094, 0.051},
		warning_text = {0.604, 0.004, 0.086},
	},
	{
		title = 'ice',
		menu_bg = {0.082, 0.231, 0.475},
		menu_text = {0.169, 0.361, 0.62},
		menu_text_disabled = {0.2, 0.345, 0.518},
		bg = {0.239, 0.471, 0.776},
		neutral = {0.294, 0.541, 0.804},
		numbers = {0.769, 0.847, 0.953},
		odd_numbers = {0.059, 0.204, 0.412},
		db4 = {0.271, 0.592, 0.882},
		goal = {0.09, 0.388, 0.827},
		goal_numbers = {0.627, 0.71, 0.831},
		indicator = {0.059, 0.204, 0.412},
		selected = {0.859, 0.925, 0.988},
		selected_numbers = {0.416, 0.537, 0.62},
		naked_bg = {0.082, 0.231, 0.475},
		naked_odd = {0.086, 0.373, 0.824},
		naked_db4 = {0.722, 0.812, 0.965},
		naked_goal = {0.627, 0.71, 0.831},
		naked_indicator = {0.086, 0.373, 0.824},
		progress = {0.918, 0.969, 1},
		help_bg = {0.718, 0.82, 0.949},
		heading = {0.224, 0.463, 0.788},
		message_bg = {0.553, 0.722, 0.922},
		message_text = {0.09, 0.388, 0.827},
		warning_bg = {0.145, 0.345, 0.604},
		warning_text = {0.918, 0.969, 1},
	},
	{
		title = 'candy',
		menu_bg = {1, .757, .749},
		menu_text = {.047, .243, .271},
		menu_text_disabled = {.722, .463, .576},
		bg = {.96, .82, .8},
		neutral = {.729, .706, .757},
		numbers = {.047, 0, .235},
		odd = {.314, .51, .635},
		db2 = {.737, .706, .282},
		db4 = {.341, .537, .341},
		goal = {.749, .453, .757},
		selected = {.627, .455, .427},
		progress = {.722, .463, .576},
		heading = {.49, .33, .3},
		message_bg = {.704, .768, .704},
		warning_bg = {.722, .463, .576},
	},
	{
		title = 'slime',
		menu_bg = {0.643, 0.514, 0.369},
		menu_text = {0.925, 0.741, 0.58},
		menu_text_disabled = {0.714, 0.596, 0.467},
		bg = {0.447, 0.388, 0.196},
		neutral = {0.62, 0.529, 0.255},
		numbers = {0.224, 0.173, 0.106},
		odd = {0.541, 0.463, 0.325},
		odd_numbers = {0.192, 0.145, 0.082},
		db4 = {0.745, 0.6, 0.102},
		db4_numbers = {0.337, 0.271, 0.047},
		goal = {0.514, 0.502, 0.078},
		selected = {0.894, 0.443, 0.329},
		selected_numbers = {0.592, 0.227, 0.173},
		naked_bg = {0.286, 0.216, 0.071},
		naked_odd = {0.541, 0.525, 0.388},
		naked_goal = {0.514, 0.541, 0.078},
		naked_indicator = {0.62, 0.529, 0.255},
		progress = {0.894, 0.682, 0.082},
		help_bg = {0.816, 0.722, 0.62},
		help_text = {0.224, 0.173, 0.106},
		heading = {0.412, 0.278, 0.11},
		message_bg = {0.314, 0.212, 0.133},
		message_text = {0.949, 0.706, 0.353},
		warning_bg = {0.749, 0.239, 0.176},
		warning_text = {0.941, 0.835, 0.773},
	},
}


function set_color(name)
	local c = colors[settings.color_scheme]
	local substitutions = {
		neutral = 'bg',
		odd = 'neutral',
		odd_numbers = 'numbers',
		db2 = 'neutral',
		db2_numbers = 'numbers',
		db4_numbers = 'numbers',
		indicator = 'neutral',
		goal_numbers = 'numbers',
		selected_numbers = 'numbers',
		naked_neutral = 'neutral',
		naked_odd = 'odd',
		naked_db2 = 'db2',
		naked_db4 = 'db4',
		naked_goal = 'goal',
		naked_indicator = 'naked_selected',
		naked_selected = 'selected',
		help_text = 'menu_text',
		message_bg = 'db4',
		message_text = 'numbers',
		warning_bg = 'odd',
		warning_text = 'message_text',
	}
	local default_colors = {
		warning_bg = {.8, .2, .2},
		message_bg = {.5, .8, .4},
	}
	love.graphics.setColor(
		c[name] or c[substitutions[name]] or
		c[substitutions[substitutions[name]]] or
		default_colors[name] or {0, 0, 0}
	)
end


function set_bg_color()
	colors.bg_color = settings.rectangles == 2 and
		colors[settings.color_scheme].naked_bg or
		colors[settings.color_scheme].bg
end


function appropriate_color(value, selected)
	if selected then
		return 'selected'
	elseif not value or value == '@' then
		return 'neutral'
	elseif value == current.goal then
		return 'goal'
	elseif value % 4 == 0 then
		return 'db4'
	elseif value % 2 == 0 then
		return 'db2'
	else
		return 'odd'
	end
end
