--[[
This file is part of Exploding numbers - a puzzle game

Exploding numbers is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
]]


menu = {
	goal = {
		text = 'Goal',
		arrow = 'ul',
		key = 'g',
		onclick = function()
			settings.menu_disabled = not settings.menu_disabled
			love.resize()
		end,
		value = function() return current.goal end
	},
	undo = {
		text = 'Undo',
		value = 'u',
		arrow = 'dl',
		key = 'u',
		validate = function(self) self.disabled = current.history.position == 0 end,
		onclick = function() undo(-1) end,
	},
	redo = {
		text = 'Solve',
		value = 'r',
		arrow = 'dr',
		key = 'r',
		validate = function(self)
			local ch = current.history
			local sol = levels[current.level].solution
			-- check to disable
			self.disabled = ch.position == #ch
			-- set text
			if not sol or #sol ~= #ch then
				self.text = 'Redo'
				return
			end
			local is_sol = true
			for s = 1, #ch do
				if not is_sol then
					self.text = 'Redo'
					return
				end
				is_sol = is_sol and
					ch[s][1] == sol[s][1] and ch[s][2] == sol[s][2]
			end
			self.text = 'Solve'
		end,
		onclick = function(self) undo(1, self.text) end,
	},
	help = {
		text = 'Help',
		value = 'F1',
		key = 'f1',
		onclick = function() help:main() end
	},
	level = {
		text = 'Level -\nF5',
		value = '',
		arrow = 'dl',
		key = 'f5',
		validate = function(self)
			self.text = current.history.position > 0 and 'Restart\nF5' or 'Level -\nF5'
		end,
		onclick = function()
			init_level(current.level - (current.history.position > 0 and 0 or 1))
			resize_grid()
		end
	},
	skip = {
		text = '+\nF6',
		arrow = 'dr',
		value = function() return level_name(current.level) end,
		common_value = true,
		key = 'f6',
		validate = function(self)
			self.text = current.history.position > 0 and 'Next\nF6' or '+\nF6'
		end,
		onclick = function()
			init_level(current.level + 1)
			resize_grid()
		end
	},
	colors_prev = {
		text = 'Color\nC -',
		value = '',
		arrow = 'dl',
		-- key is handled globally
		onclick = function()
			local max_colors = get_progress() < 1 and (#colors - colors.bonus) or #colors
			settings.color_scheme = (settings.color_scheme - 2) % max_colors + 1
			set_bg_color()
			screen.changed = true
		end
	},
	colors_next = {
		text = 'scheme   \n       c +',
		value = function () return colors[settings.color_scheme].title end,
		common_value = true,
		arrow = 'dr',
		-- key is handled globally
		onclick = function()
			local max_colors = get_progress() < 1 and (#colors - colors.bonus) or #colors
			settings.color_scheme = settings.color_scheme % max_colors + 1
			set_bg_color()
			screen.changed = true
		end
	},
	reset_toggle = {
		text = 'Reset',
		value = 'F10',
		arrow = 'ur',
		key = 'f10',
		onclick = function(self)
			if self.value == 'n' then
				menu.order = menu.settings_order
				self.text = 'Reset'
				self.value = 'F10'
				self.key = 'f10'
				self.arrow = 'ur'
				screen.messages = nil
				resize_menu()
			else
				menu.order = menu.reset_order
				self.text = 'reset?'
				self.value = 'n'
				self.key = 'n'
				self.arrow = nil
				resize_menu()
				new_message('This will delete settings, custom and solved levels', true, 500)
			end
		end
	},
	confirm_reset = {
		text = '  Confirm',
		value = 'y',
		key = 'y',
		onclick = function()
			del_game()
			current = nil
			love.event.push('quit')
		end
	},
	resize_buttons = {
		text = 'Buttons',
		value = function() return settings.menu_size end,
		key = 'b',
		onclick = function()
			settings.menu_size = (settings.menu_size == 60) and 100 or 60
			resize_menu_fonts()
			love.resize()
		end
	},
	change_shape = {
		text = 'Shape',
		value = 'o',
		key = 'o',
		validate = function(self) self.disabled = get_progress() < .75 end,
		onclick = function()
			settings.rectangles = ((settings.rectangles or 0) + 1) % 3
			draw_playfield_shape = switch_playfield_shape(settings.rectangles)
			set_bg_color()
		end
	},
	fullscreen = {
		text = 'Fullscr.',
		value = function() return screen.fullscreen and 'yes' or 'no' end,
		key = 'f',
		arrow = 'ul',
		onclick = function() toggle_fullscreen() end
	},
	settings_toggle = {
		text = 'Settings',
		value = 's',
		key = 's',
		arrow = 'ur',
		onclick = function(self)
			settings.menu_disabled = false
			menu.order = (menu.order == menu.settings_order) and
				menu.default_order or menu.settings_order
			self.text = (menu.order == menu.settings_order) and 'Return' or 'Settings'
			resize_menu()
		end
	},
	decrease_min_zoom = {
		text = '  Minimal\n  F2 -',
		value = '',
		arrow = 'dl',
		key = 'f2',
		onclick = function(self)
			-- prevent dragging while zooming out
			-- as it creates bad offsets
			screen.grid_drag[1] = nil
			resize_min_grid_by(- 10)
			resize_grid()
			screen.zoom_indicator_delay = 100
		end
	},
	increase_min_zoom = {
		text = ' zoom\n   F3 +',
		value = function() return settings.min_grid or 0 end,
		arrow = 'dr',
		common_value = true,
		key = 'f3',
		onclick = function(self)
			resize_min_grid_by(10)
			resize_grid()
			screen.zoom_indicator_delay = 100
		end
	}
} -- end of menu table


menu.default_order = {
	menu.goal, menu.level, menu.skip,
	menu.undo, menu.redo,
	menu.settings_toggle, menu.help
}
menu.settings_order = {
	menu.fullscreen,
	menu.colors_prev, menu.colors_next,
	menu.change_shape, menu.resize_buttons,
	menu.decrease_min_zoom, menu.increase_min_zoom,
	menu.reset_toggle, menu.settings_toggle
}
menu.reset_order = {
	menu.confirm_reset, menu.reset_toggle
}
menu.order = menu.default_order


function exit_menu_or_quit()
	if menu.order == menu.settings_order then
		menu.settings_toggle:onclick()
	elseif menu.order == menu.reset_order then
		menu.reset_toggle:onclick()
	elseif menu.order == menu.edit_order then
		menu.edit_stop:onclick()
	elseif
		screen.messages and
		screen.messages[#screen.messages].message == 'Press again to quit'
	then
		love.event.push("quit")
	else
		screen.messages = nil
		new_message('Press again to quit', true)
	end
	screen.changed = true
end


function resize_menu_disabled_button()
	-- resizes on goal change and menu font size change
	screen.menu_disabled_button = {
		x = 4,
		y = 4,
		w = menu_font_large:getWidth(current.goal),
		h = menu_font_large:getHeight(current.goal),
		ww = math.max(settings.menu_size, menu_font_large:getWidth(current.goal))
	}
	screen.changed = true
end


function try_disabling_menu_items()
	for a = 1, #menu.order do
		if menu.order[a].validate then menu.order[a]:validate() end
	end
	if screen then screen.changed = true end
end


function resize_menu_fonts()
	if menu_font_small then menu_font_small:release() end
	if menu_font_large then menu_font_large:release() end
	menu_font_small = love.graphics.newFont(settings.menu_size / 4)
	menu_font_large = love.graphics.newFont(settings.menu_size / 2)
	resize_menu_disabled_button()
end


function resize_menu()
	screen.top_margin = (settings.menu_disabled and 0) or settings.menu_size
	if not (menu_font_small and menu_font_large) then resize_menu_fonts() end
	screen.buttons = {}
	for button = 1, #menu.order do
		local x = (button - 1) * settings.menu_size
		screen.buttons[button] = {
			x = x, y = 0, w = settings.menu_size, h = settings.menu_size,
			text_x = x + (settings.menu_size - menu_font_small:getWidth(menu.order[button].text)) / 2,
			text_y = settings.menu_size * .1,
			text2_y = settings.menu_size - menu_font_large:getHeight(current.level)
		}
	end
	screen.buttons.dx = math.max(0,
		(screen.w - #screen.buttons * settings.menu_size) / 2
	)
end


function shift_menu(new_dx)
	local dw = screen.w - #screen.buttons * settings.menu_size
	screen.buttons.dx = math.max(
		math.min(0, new_dx),
		math.min(dw / 2, dw)
	)
	screen.changed = true
end


function draw_menu()
	-- a small button which opens menu
	if settings.menu_disabled then
		love.graphics.setFont(menu_font_large)
		local s = screen.menu_disabled_button
		local function poly(style, c)
			set_color(c)
			love.graphics.polygon(style,
				s.x, s.y,
				s.x + s.ww, s.y,
				s.x + s.ww + s.h / 4, s.y + s.h / 2,
				s.x + s.ww, s.y + s.h,
				s.x, s.y + s.h
			)
		end
		poly('fill', 'menu_bg')
		love.graphics.setLineWidth(1)
		poly('line', 'menu_text')
		love.graphics.print(current.goal, 1.5 + s.x + (s.ww - s.w) / 2, s.y)
		-- hint
		love.graphics.setFont(menu_font_small)
		set_color('menu_text_disabled')
		love.graphics.print('g', 2 + s.x, s.y)
		return
	end
	-- the menu
	love.graphics.setScissor(screen.x, screen.y, screen.w, screen.h)
	set_color('menu_bg')
	love.graphics.rectangle('fill',
		0, 0,
		screen.w, screen.top_margin
	)
	local dx = screen.buttons.dx
	for a = 1, #menu.order do
		local s = screen.buttons[a]
		if menu.order[a].color then
			set_color(menu.order[a].color)
			love.graphics.rectangle('fill', dx + s.x, s.y, s.w, s.h)
		end
		if menu.order[a].arrow then
			c = s.h / 2.83 -- 2 * sqrt(2)
			local coordinates = {
				ul = {
					dx + s.x, s.y,
					dx + s.x + s.h / 2, s.y,
					dx + s.x, s.y + s.h / 2
				},
				ur = {
					dx + s.x + s.w, s.y + s.h / 2,
					dx + s.x + s.w, s.y,
					dx + s.x + s.w - s.h / 2, s.y
				},
				dr = {
					-1 + dx + s.x + s.w, s.y + s.h / 2,
					-1 + dx + s.x + s.w, s.y + s.h,
					-1 + dx + s.x + s.w - s.h / 2, s.y + s.h
				},
				dl = {
					dx + s.x, s.y + s.h / 2,
					dx + s.x, s.y + s.h,
					dx + s.x + s.h / 2, s.y + s.h
				},
				l = {
					dx + s.x, s.y + s.h / 2,
					dx + s.x + c, s.y + s.h / 2 - c,
					dx + s.x + c, s.y + s.h / 2 + c,
				},
				r = {
					dx + s.x + s.w, s.y + s.h / 2,
					dx + s.x + s.w - c, s.y + s.h / 2 - c,
					dx + s.x + s.w - c, s.y + s.h / 2 + c,
				}
			}
			set_color('menu_text_disabled')
			love.graphics.polygon('fill', coordinates[menu.order[a].arrow])
		end
		local disabled = menu.order[a].disabled
		set_color('menu_text' .. (disabled and '_disabled' or ''))
		love.graphics.setFont(menu_font_small)
		love.graphics.print(menu.order[a].text, dx + s.text_x, s.text_y)
		local v = menu.order[a].value
		if v then
			disabled = disabled and
				(not menu.order[a].common_value or menu.order[a - 1].disabled)
			set_color('menu_text' .. (disabled and '_disabled' or ''))
			if type(v) == 'function' then v = v() end
			love.graphics.setFont(menu_font_large)
			local dx = menu.order[a].common_value and dx - settings.menu_size or dx
			local sw = menu.order[a].common_value and 2 * s.w or s.w
			love.graphics.printf(v, dx + s.x, s.text2_y, sw, 'center')
		end
	end
	love.graphics.setScissor()
end
